.include "vtuilib-ca65.inc"
.import _set_pallete_hue
.import pusha
.import popa, popax
.import _waitvsync
.import _set_text_256color
.import _backup_pallete, _restore_pallete
.import strlen
.export _update_pallete
.export _pallete_h
.export _cats_base
.export _cats_len
.export _draw_string
CINT := $FF81
CHROUT := $FFD2
screen_mode := $FF5F
.segment "ONCE"
.segment "CODE"
start:
	jsr vtui_initialize ; Init VTUI
	lda #$8E ; Switch to UP/GFX charset
	jsr CHROUT
	sec ; Get the screen mode and dimensions
	jsr screen_mode
	sta _videomode ; Store the screen mode and dimensions
	stx _screen_w
	sty _screen_h
	jsr _backup_pallete
	lda _videomode ; Reload the video mode since _backup_pallete likely clobbered it
	jsr vtui_screen_set ; Set the screen mode through VTUI so that it knows about the screen mode.
	bcs exit ; If this failed, exit
	lda #1 ; Turn on 256 color text
	jsr _set_text_256color
	sec ; Set the bank to 1
	jsr vtui_set_bank
	lda #1 ; Set the stride to 1 also
	jsr vtui_set_stride
	clc ; Set decrement to false
	jsr vtui_set_decr
	lda #' ' ; Clear the screen with spaces
	ldx #0
	jsr vtui_clr_scr
	lda #<_string1 ; Load the first string
	sta r0l
	lda #>_string1
	sta r0h
	lda #0 ; ... and display it at (0, 0)
	sta r1l
	lda #0
	sta r2l
	jsr _draw_string ; ... by calling _draw_string
	lda #<_string2 ; Load the next string
	sta r0l
	lda #>_string2
	sta r0h
	lda #0 ; ... display it at (0, 1)
	sta r1l
	lda #1
	sta r2l
	jsr _draw_string ; ... by calling the correct routine
	lda #<_string3 ; Load the 3rd string
	sta r0l
	lda #>_string3
	jsr strlen ; .. calculate it's length for right alignment
	sty r4l
	lda _screen_w ; ... Then load the screen width and subtract the string length - 1
	clc
	sbc r4l
	inc
	sta r1l ; And set up for display at this X value
	lda _screen_h ; Load the screen height for bottom alignment
	dec ; Decrement it because it's 1 tile off screen
	tay ; And set it up for the VTUI library
	lda r1l ; Now reload the X value
	jsr vtui_gotoxy ; Go to the right position
	lda #0 ; Enable conversion to string codes
	ldx #1 ; Set the color to white
	ldy r4l ; Load the length of the string
	jsr vtui_print_str ; and print the string
@loop:
	jsr _waitvsync ; Avoid updating the pallete too fast
	jsr _update_pallete ; Update the pallete
	jsr $FFE4 ; KERNAL GETIN
	beq @loop ; If zero, there's no keys pressed
	cmp #$1B  ; Compare with ESC keycode
	bne @loop ; If not ESC, don't exit yet
exit:
	jsr CINT ; Restore the VERA's state
	clc ; Restore the video mode
	lda _videomode
	jsr screen_mode
	rts ; Exit the program
_update_pallete:
	lda _pallete_h ; Load the previous hue
	inc ; Increment it
	jsr make_hue_valid ; Make sure it stays within bounds
	sta _pallete_h ; Store it
	sta _pallete_hue_tmp
	ldx #0 ; Initialize the for loop
@pallete_set_loop:
	lda _pallete_hue_tmp ; Add the difference to the hue
	adc _cats_diff
	jsr make_hue_valid ; ... while making sure it stays valid
	sta _pallete_hue_tmp ; And store the result
	txa ; Copy the index
	clc
	adc _cats_base ; Add the pallete index base
	sta r0l ; Set the parameter
	lda _pallete_hue_tmp ; Load the pallete index
	phx ; Back up the index
	jsr _set_pallete_hue ; Set the hue
	plx ; Restore the index
	inx ; Increment the index
	cpx _cats_len ; Compare the index against the max
	bne @pallete_set_loop ; x < _cats_len
@done:
	rts

make_hue_valid:
	cmp #192 ; Check if the hue is in the correct range before modifying it
	bcc @hue_valid ; If it is, there's no need to change it
	clc ; Otherwise, a=a-(max-1), because otherwise we're off by one
	sbc #191
@hue_valid:
	rts
_draw_string:
	lda r2l ; Load the Y parameter
	sta _string_pos+1 ; Store the Y parameter
	lda r1l ; Load the X parameter
	sta _string_pos ; Store the X parameter
	ldy _string_pos+1 ; Load the Y param into the Y register
	jsr vtui_gotoxy ; Go to the right position
	ldy #0 ; Initialize the index
@loop:
	lda (r0),y ; Load the character
	beq @end ; If it's zero, we're done
	tya ; Copy the index to a so that the pallete index can be calculated
	clc
	adc _string_pos+1 ; Add the Y value
	clc
	adc _string_pos ; Add the X value
	cmp _cats_len ; Compare the calculated pallete index against the pallete max index
	bcc @after_set_cats_len ; If (pidx < _cats_len) goto @after_set_cats_len
	inc ; Prevent an off-by-one error
	sta _cats_len ; Update the max pallete index
@after_set_cats_len:
	lda (r0),y ; Load the character again
	jsr vtui_pet2scr ; Convert it to screen codes
	sta _char_scrcode ; Store it for later
	lda _cats_base ; Load the base pallete index
	sty _pallete_i ; Copy Y to _pallete_i
	clc
	adc _string_pos ; Add the string pos X to a
	clc
	adc _string_pos+1 ; Add the string pos Y to a
	clc
	adc _pallete_i ; Add the pallete index to a
	tax ; Copy a to x
	lda _char_scrcode ; Load the screencode
	jsr vtui_plot_char ; Plot the character
	iny ; Increment the index
	jmp @loop ; Go back to beginning of loop for the next character
@end:
	rts

.segment "DATA"
_pallete_h:
	.byte 0
_cats_len:
	.byte 0
_cats_diff:
	.byte 8
_cats_base:
	.byte 17
_string1:
	.byte "this text is rainbow",0
_string2:
	.byte "press [esc] to exit.",0
_string3:
	.byte "written in assembly.",0
.segment "BSS"
_hue_tmp:
	.res 2
_pallete_hue_tmp:
	.res 1
_pallete_i:
	.res 1
_string_pos:
	.res 2
_char_scrcode:
	.res 1
_videomode:
	.res 1
_screen_w:
	.res 1
_screen_h:
	.res 1