.include "vtuilib-ca65.inc"
VERA_L1_CONFIG = $9F34
.export _backup_pallete
.export _restore_pallete
.export _set_pallete_color
.export _set_pallete_hue
.export _set_text_256color
.import pusha
.import popa
.import popax
.import addysp1
.import tosdivax
.importzp sp
.segment "CODE"
_set_pallete_color:
	tax
	lda VERA_CTRL
	and #%11111110
	sta VERA_CTRL
	lda #$FA
	sta VERA_ADDR_M
	txa
	clc
	rol
	sta VERA_ADDR_L
	bcc @carry_not_set
	lda VERA_ADDR_M
	inc
	sta VERA_ADDR_M
@carry_not_set:
	lda #%00010001
	sta VERA_ADDR_H
	lda r0l ; Load blue parameter
	and #$0F ; Make it only include valid bits
	sta new_color ; Store the value
	lda r1l; Load green parameter
	and #$0F ; Remove invalid bits
	clc
	rol ; Bit shift left 4 times
	rol
	rol
	rol
	ora new_color ; Add to color
	sta new_color ; Store as part of color
	lda r2l
	sta new_color+1
	lda new_color
	sta VERA_DATA0
	lda new_color+1
	sta VERA_DATA0
	rts
_set_pallete_hue:
	clc ; Divide by 32
	ror
	tax ; Copy A to X once 2 is divided
	lsr
	lsr
	lsr
	lsr
	clc
	tay ; Modulo a by 6 using lookup table
	lda mod6,y
	pha ; Backup a
	txa ; AND x with 15
	and #15
	tax
	pla ; Restore a
	stx hue_tmp+3 ; Copy X to hue_tmp+3
	cmp #0
	beq @case0
	cmp #1
	beq @case1
	cmp #2
	beq @case2
	cmp #3
	beq @case3
	cmp #4
	beq @case4
	jmp @case5
@case0:
	ldy #15
	sty hue_tmp
	stx hue_tmp+1
	sta hue_tmp+2 ; A can only be 0 here
	jmp @end
@case1:
	lda #15
	sta hue_tmp+1
	sec
	sbc hue_tmp+3
	sta hue_tmp
	lda #0
	sta hue_tmp+2
	jmp @end
@case2:
	lda #0
	sta hue_tmp
	lda #15
	sta hue_tmp+1
	stx hue_tmp+2
	jmp @end
@case3:
	lda #15
	sta hue_tmp+2
	sec
	sbc hue_tmp+3
	sta hue_tmp+1
	lda #0
	sta hue_tmp
	jmp @end
@case4:
	stx hue_tmp
	lda #0
	sta hue_tmp+1
	lda #15
	sta hue_tmp+2
	jmp @end
@case5:
	lda #15
	sta hue_tmp
	sec
	sbc hue_tmp+3
	sta hue_tmp+2
	lda #0
	sta hue_tmp+1
@end:
	ldx r0l
	lda hue_tmp
	sta r0l
	lda hue_tmp+1
	sta r1l
	lda hue_tmp+2
	sta r2l
	txa
	jsr _set_pallete_color
	rts

_backup_pallete:
	lda VERA_CTRL
	and #%11111110
	sta VERA_CTRL
	lda #$FA
	sta VERA_ADDR_M
	lda #$00
	sta VERA_ADDR_L
	lda #%00010001
	sta VERA_ADDR_H
	ldx #$00
@part1:
	lda VERA_DATA0
	sta pallete_backup,x
	inx
	bne @part1
@part2:
	lda VERA_DATA0
	sta pallete_backup+256,x
	inx
	bne @part2
@end:
	rts
_restore_pallete:
	lda VERA_CTRL
	and #%11111110
	sta VERA_CTRL
	lda #$FA
	sta VERA_ADDR_M
	lda #$00
	sta VERA_ADDR_L
	lda #%00010001
	sta VERA_ADDR_H
	ldx #$00
@part1:
	lda pallete_backup,x
	sta VERA_DATA0
	inx
	bne @part1
@part2:
	lda pallete_backup+256,x
	sta VERA_DATA0
	inx
	bne @part2
@end:
	rts
_set_text_256color:
	beq @false
@true:
	lda VERA_L1_CONFIG
	ora #%1000
	sta VERA_L1_CONFIG
	rts
@false:
	lda VERA_L1_CONFIG
	and #%11110111
	sta VERA_L1_CONFIG
	rts
.segment "DATA"
mod6:
	.byte 0,1,2,3,4,5,0,1,2,3,4,5,0,1,2,3,4,5,0,1,2,3,4,5
.segment "BSS"
pallete_backup:
	.res 512
new_color:
	.res 2
hue_tmp:
	.res 4